% Entry point (Hit F5 to run)
%{
===============================================================================
DESIGNED BY:    Tsezar F. Seman
AFFILIATION:    NJIT University, Physics Department
MODIFIED ON:    12/16/2012
COPYRIGHT:      (c) Tsezar F. Seman. All rights reserved.
PURPOSE:        Plotting experimental RIXS data.
===============================================================================
%}
function plot_expcontour()
	clc; addpath('../../_common_');
	
	d = LoadData();
	isLorenzOn = 0;
	ContourPlot(20, d, isLorenzOn);
	if (isLorenzOn)
		% w/ Lorenzian:
		fig2eps(gcf,'exper_pol_Lor', 1);	% 0-vector, 1-image
	else
		% w/o Lorenzian:
		fig2eps(gcf,'exper_pol', 1);	% 0-vector, 1-image
	end
	
	
end
function ContourPlot(figId, d, isLorenzOn)
	
	gammaW = 0.15;
	FontSize = 15;
	Ny = 100;
	Nx = 50;
	
	LorentzianW = @(ww,w0) ( (gammaW./pi) ./ ((ww-w0).^2 + gammaW.^2) );
	
	minmax_H = [0, 0.7];
	minmax_E = [0, 5];
	%figure(1); clf; hold all;
	
	
	% horizontal interpolation:
	polyN = 5;
	nE = length(d.E);
	mI = zeros(nE, Nx);
	[H,iH] = sortrows( [d.Hr; d.Hb; d.Hk] );
	x = linspace(minmax_H(1),minmax_H(2), Nx);
	for iE = 1:nE
		% collect intensities:
		I = [d.Ir(iE,:), d.Ib(iE,:), d.Ik(iE,:)]';
		I = I(iH,1);
		% poly interpolate I values:
		p = polyfit(H,I, polyN);
		mI(iE,:) = polyval(p,x);
		
%		plot(H, I,'.-b')
%		plot(x, mI(iE,:),'.-r')
	end
	
	% vertical Lorenzian:
	mI_L = zeros(Ny,Nx);
	y = linspace(minmax_E(1), minmax_E(2), Ny);
	for ix = 1:Nx
		% collect all intensities and energies for a given k-point
		I_tmp = zeros(size(y));
		for iE = 1:nE
			I_tmp = I_tmp + mI(iE,ix) .* LorentzianW(y, d.E(iE,1));
		end
		mI_L(:,ix) = I_tmp';
	end
	
	
	figure(figId); clf;
	hold on;
	
	if (isLorenzOn)
		% w/ Lorenzian:
		[mH,mE] = meshgrid(x, y);
		pcolor(mH,mE, mI_L);
	else
		% w/o Lorenzian:
		[mH,mE] = meshgrid(x, d.E);
		pcolor(mH,mE, mI);
	end
	
	%caxis([0, 1]);
	axis('tight');	%'equal'
	colormap(hot(128));
	shading('flat');
	hcb = colorbar;
	%xlim(minmax_H);
	xlim([0,0.5]);
	ylim(minmax_E);
	
	
	% axis matters
	ylabel('Energy (eV)', 'FontSize',FontSize);
	set(gca, 'TickDir','out', 'FontSize',FontSize);
	set(gca, 'XTick',[0,0.25,0.5,0.75]);
	set(gca, 'XTickLabel',{'0','','0.5',''});
	set(gca, 'YTick',0:1:5);
	set(gca, 'LineWidth',2);
	box('on')
	set(gca,'PlotBoxAspectRatio',[.4333 1 1])
	
	% plot vertical white line
%	plot(0.5*[1,1], minmax_E, '--w', 'LineWidth',1.5);
	
	hold off;
	%max(max(mI))
end

function d = LoadData()
	data = load('data_red.txt');
	Hr = load('data_red_H.txt');
	E = data(:,1);
	Ir = data(:,2:end);
	
	data = load('data_blu.txt');
	Hb = load('data_blu_H.txt');
	Ib = data(:,2:end);
	
	data = load('data_blk.txt');
	Hk = load('data_blk_H.txt');
	Ik = data(:,2:end);
	
	d.E = E;
	d.Hr = abs(Hr');
	d.Ir = SubBkg(E,Ir);
	d.Hb = abs(Hb');
	d.Ib = SubBkg(E,Ib);
	d.Hk = abs(Hk');
	d.Ik = SubBkg(E,Ik);
	d.maxI = max([max(d.Ir), max(d.Ib), max(d.Ik)]);
	d.minI = min([min(d.Ir), min(d.Ib), min(d.Ik)]);
end
% subtract Lorenzian background:
function y = SubBkg(x,y)
	% background params:
	A = 1600;
	gamma_bk = 1.5;
	x0 = 4.5;
	LorentzianX = @(xx,x0) A*( (gamma_bk./pi) ./ ((xx-x0).^2 + gamma_bk.^2) );
	yLor = LorentzianX(x,x0);
	ny = size(y,2);
	for i = 1:ny
		y(:,i) = y(:,i) - yLor;
	end
end























