%{
===============================================================================
DESIGNED BY:    Tsezar F. Seman
AFFILIATION:    NJIT University, Physics Department
MODIFIED ON:    06/16/2012
COPYRIGHT:      (c) Tsezar F. Seman. All rights reserved.
PURPOSE:        Function initializes all static components, alocates all global
				variables. This function must be called ONCE at the begining.
INPUT:			N - system base => N^2 atoms
				CH - flag for Core-Hole presence
===============================================================================
%}
function Initialize(t0,lambda)
	
	%------------------------------------------------------
	% DEFINE Eg GLOBAL STRUCTURE
	%------------------------------------------------------
	
	global Eg
	% Physical Constants:
	%Eg.C.lambda = sqrt(3/2)*1.38*0.4/0.228;		% (eV/angstrom), J-T param, (= 2.9652)
%	Eg.C.lambda = 1.69;							% (eV/angstrom), J-T param
%	Eg.C.JhSc = 2.47 / 2;						% (eV), Hund's param, (= 1.235)
%	Eg.C.U = 1.6;								% (eV), Coulomb potential
%	Eg.C.Uch = -2.6;							% (eV), core-hole potential

	Eg.C.beta = 1.5;							% (unitless) breathing parameter
	Eg.C.Q_1s =  0.05311622;					% (A) distortion prarmenter
	Eg.C.Q_2s =  0.05444722;					% (A) distortion prarmenter
	Eg.C.Q_3u =  0.10736930;					% (A) distortion prarmenter
	Eg.C.Q_3s = -0.01183920;					% (A) distortion prarmenter
	
	
	% CHOSEN PARAMETERS:
	Eg.C.t0 = t0;
	Eg.C.lambda = lambda;
	Eg.C.U = 0.95 * 1.6;
	Eg.C.JhSc = 1.8 * 2.47 / 2;
	Eg.C.Uch = -2.0;


	% NOTE:
	% All Eg.C.shift paramters must be adjusted through running run4_plot_band()
	switch Eg.C.t0
		case 0.1
			Eg.C.shift = 1.59346458;	% based on N8
		case 0.6
			Eg.C.shift = 1.84194376;	% based on N8
		case 1.1
			Eg.C.shift = 2.14013226;	% based on N8
		case 1.4
			Eg.C.shift = 3.13399009;	% based on N16   XXX
		%-----------------------------------------------
		case 0.9
			Eg.C.shift = 3.20777736;	% based on N8   XXX
		case 1.0
			Eg.C.shift = 3.10568240;	% based on N8   XXX
		case 1.2
			Eg.C.shift = 3.01260735;	% based on N8   XXX
		case 1.3
			Eg.C.shift = 3.06619225;	% based on N8   XXX
		%-----------------------------------------------
		otherwise
			Eg.C.shift =  2.16062741;	% based on N8   XXX
	end

	% Enable ONLY for running: run_Uch() and Uch_Estimation()
	%Eg.C.Uch = -1 * load('_temp_.txt');
	
	%------------------------------------------------------
    % Hamiltonians 16x16:
	Eg.H_hund = SetHund_eg(Eg.C.JhSc);
	Eg.H_jt = SetJahnTeller_eg(Eg.C.lambda, Eg.C.Q_3u, Eg.C.Q_3s, Eg.C.Q_2s);
	Eg.H_brtng = SetBreathing_eg(Eg.C.lambda,Eg.C.beta, Eg.C.Q_1s);
	Eg.H_shift = Eg.C.shift .* eye(16);
	
	%------------------------------------------------------
	% FOR BANDS ONLY
	%------------------------------------------------------
%	if not(isCluster)
%		Eg.H_coloumb = SetCoulomb_Bands_eg(N);
%	end
end
%==========================================================
function m16 = SetHund_eg(JhSc)
	Z = zeros(2);
	G = (JhSc / 2) * eye(2);
	m16 = [ ...
		 Z, G,-G, Z,   Z,-G,-G, Z; ...
		 G, Z, Z,-G,  -G, Z, Z,-G; ...
		-G, Z, Z, G,  -G, Z, Z,-G; ...
		 Z,-G, G, Z,   Z,-G,-G, Z; ...
		 Z,-G,-G, Z,   Z, G,-G, Z; ...
		-G, Z, Z,-G,   G, Z, Z,-G; ...
		-G, Z, Z,-G,  -G, Z, Z, G; ...
		 Z,-G,-G, Z,   Z,-G, G, Z ];
	m16(:,:,2) = -m16(:,:,1);
end

function m16 = SetJahnTeller_eg(lambda, Q_3u, Q_3s, Q_2s)
	Z = zeros(2);
	A = -lambda * Q_3u * [ 1,  0;  0, -1];
	B = -lambda * Q_3s * [ 1,  0;  0, -1];
	C = -lambda * Q_2s * [ 0, -1; -1,  0];
	m16 = [ ...
		A, Z, Z, B,   C, Z, Z, C; ...
		Z, A, B, Z,   Z, C, C, Z; ...
		Z, B, A, Z,   Z, C, C, Z; ...
		B, Z, Z, A,   C, Z, Z, C; ...
		C, Z, Z, C,   A, Z, Z, B; ...
		Z, C, C, Z,   Z, A, B, Z; ...
		Z, C, C, Z,   Z, B, A, Z; ...
		C, Z, Z, C,   B, Z, Z, A ];
end

function m16 = SetBreathing_eg(lambda, beta, Q_1s)
	Z = zeros(2);
	A = -lambda * beta * Q_1s * [1, 0; 0, 1];
	m16 = [ ...
		Z, Z, Z, A,   Z, Z, Z, Z; ...
		Z, Z, A, Z,   Z, Z, Z, Z; ...
		Z, A, Z, Z,   Z, Z, Z, Z; ...
		A, Z, Z, Z,   Z, Z, Z, Z; ...
		Z, Z, Z, Z,   Z, Z, Z, A; ...
		Z, Z, Z, Z,   Z, Z, A, Z; ...
		Z, Z, Z, Z,   Z, A, Z, Z; ...
		Z, Z, Z, Z,   A, Z, Z, Z ];
end
%==========================================================
%{
Creates a pair of 16x16 matrices used in Oper_n()
First 16x16 matrix is for kx component, second is for ky
%}
function m16z2 = Get_kDiff_eg()
	m16z2 = zeros(16,16,2);
	m2 = ones(2);
	del_kx = [0, pi,  0, pi,   -pi/2,  pi/2, -pi/2, pi/2];
	del_ky = [0,  0, pi, pi,   -pi/2, -pi/2,  pi/2, pi/2];
	for ir = 1:8
		iir = (ir-1)*2+1;
		for ic = 1:8
			iic = (ic-1)*2+1;
			m16z2(iir:iir+1, iic:iic+1, 1) = (del_kx(ic) - del_kx(ir)) .* m2;
			m16z2(iir:iir+1, iic:iic+1, 2) = (del_ky(ic) - del_ky(ir)) .* m2;
		end
	end
end

%{
Creates 3 pairs of 16x16 matrices used  in Oper_n()
There are 3 atom types {1,2,3}, and two orbital types each {-,+}
%}
function m16_R = Set_R_eg(Q_3u, Q_3s, Q_2s)
	if abs(Q_2s) > eps
		Q_33 = Q_3u + Q_3s;
		th0 = atan( (-Q_33 + sqrt(Q_33^2 + 4*Q_2s^2)) / (2*Q_2s) );
		theta = [-th0, 0, th0];
	else
		theta = [0, 0, 0];
	end
	
	% 1-2 dim is 16x16
	% 3-rd dim is atom type based on theta {1,2,3}	=> {Mn3_x, Mn4, Mn3_y}
	% 4-th dim is orbital {1,2} => {-,+}
	m16_R = zeros(16,16,3,2);
	
	c2 = cos(theta).^2;
	s2 = sin(theta).^2;
	cs = cos(theta).*sin(theta);

	ind = 1:2:16;
	for i = 1:3
		m16_R(ind,ind,		i,1) =  c2(i);
		m16_R(ind,ind+1,	i,1) =  cs(i);
		m16_R(ind+1,ind,	i,1) =  cs(i);
		m16_R(ind+1,ind+1,	i,1) =  s2(i);
		
		m16_R(ind,ind,		i,2) =  s2(i);
		m16_R(ind,ind+1,	i,2) = -cs(i);
		m16_R(ind+1,ind,	i,2) = -cs(i);
		m16_R(ind+1,ind+1,	i,2) =  c2(i);
	end
end
%==========================================================
% FOR BANDS ONLY:
function H_coulomb = SetCoulomb_Bands_eg(N_base)
	global GL  Eg
	
	NN = N_base^2;
	iPoints = []; nXpt_list = [];
	load(sprintf('%s/_main_/N%02d_CH0__nXpt_list',GL.path,N_base));
	nP = size(iPoints,1);
	
	% Create U-based list for Hartree-Fock
	U_list = zeros(size(nXpt_list));
	nXpt_sum = Eg.C.U * sum(nXpt_list(:,1:4),2);
	for ic = 1:4
		U_list(:,ic) = nXpt_sum - Eg.C.U * nXpt_list(:,ic);
	end
	
	m16_R = Set_R_eg(Eg.C.Q_3u, Eg.C.Q_3s, Eg.C.Q_2s);
	kDiff_eg = Get_kDiff_eg();
	
	spUp = 1; spDn = 2;
	oMn = 1; oPl = 2;
	H_coulomb = zeros(16,16,2);	
	for i = 1:nP
		U = U_list(i,:);
		vi = iPoints(i,1:2);
		iType = iPoints(i,3);
		m16_Exp = (1/NN) .* exp(1i.*( vi(1) .* kDiff_eg(:,:,1) + vi(2) .* kDiff_eg(:,:,2) ));
		
		H_coulomb(:,:,spUp) = H_coulomb(:,:,spUp) + ...
			m16_Exp .* ( U(1) .* m16_R(:,:,iType,oMn) + U(2) .* m16_R(:,:,iType,oPl) );
		
		H_coulomb(:,:,spDn) = H_coulomb(:,:,spDn) + ...
			m16_Exp .* ( U(3) .* m16_R(:,:,iType,oMn) + U(4) .* m16_R(:,:,iType,oPl) );
	end
end
%==========================================================

































