% Entry point (Hit F5 to run)
%{
===============================================================================
DESIGNED BY:    Tsezar F. Seman
AFFILIATION:    NJIT University, Physics Department
MODIFIED ON:    06/16/2012
COPYRIGHT:      (c) Tsezar F. Seman. All rights reserved.
PURPOSE:        Program calculates various Hamiltonians for LaSrMnO4 using 
                tight binding model.
===============================================================================
%}
function run1_HF(N,CH, bReloadLastState)
	clc; addpath('_functions_','_common_');
	%------------------------------------------------------
	% STARTING PARAMS:
	if nargin < 1
		clear all;
		N = 4;
		CH = 0;
	end
	if nargin < 3
		% 0:start from scratch, 1:reload from last state, 2:plot quickly
		bReloadLastState = 0;
	end
	maxSteps = 300;
	%------------------------------------------------------
	% INITIALIZATION:
	if bReloadLastState
		% STARTING FROM SAVED DATA:
		CmdTitle('RE-LOADING SAVED DATA FOR LaSrMnO4');
		LoadLastState(N,CH);
		fprintf('%sSYSTEM: N=%i, CH=%i\n\n', StageId(), N,CH);
	else
		% STARTING FROM SCRATCH:
		CmdTitle('LaSrMnO4 HAMILTONIAN CALCULATION');
		Initialize(N,CH);
		fprintf('%sSYSTEM: N=%i, CH=%i\n', StageId(), N,CH);
		fprintf('\n%sCONSTRUCTING OPERATORS:... \n\n', StageId());
		Set_nOper_All();	% set all n operators for entire i-Space
		Set_H_nonint();
		Set_H_corehole();
	end
	
	if bReloadLastState < 2
		%------------------------------------------------------
		% CORE ALGORITHM:
		% converging toward stable <n> values
		ConvergeData(maxSteps, bReloadLastState);

		%------------------------------------------------------
		% SAVE MISC DATA
		SaveLastState();	% save all the final data

		fprintf('\n%sSAVING TABLES: ...\n', StageId());
		Table_nXpt();
		Table_SysParams();

		%------------------------------------------------------
		% EXPORT DATA:
		%{-
		fprintf('\n%sEXPORING DATA FOR RIXS AND nDIFF: ...\n', StageId());
		Save_nDiff();	% For nDiff analysis
		Save_RIXS();		% For RIXS analysis
		
		Save_Hcoloumb_Bands();
		%}
	end
	
	% use ONLY for Uch estimation:
	%ExtractDataForUch();

	GetEocc();

	%------------------------------------------------------
	% PLOTS:
	%{
	fprintf('\n%sPLOTTING VARIOUS DATA: ...\n', StageId());
%	Plot_Energies();
	Plot_Tracking();
	
%	Plot_NetSpin();
%	Plot_NetCharge();
	%}
end

%==========================================================
% KEY ALGORITHMIC COMPONNETS:

% Construct all <n> operators
function Set_nOper_All()
	% ACCESS:
	global GL
	
	% Gather all n_Operaors and save them for later use
	nPt = size(GL.iPoints,1);
	for iPt = 1:nPt
		szFile = sprintf(GL.file_nOper, iPt);
		if exist(szFile, 'file') ~= 2
			c_nOper = cell(1,2);
			% Eg:
			nOper_eg = Oper_n_eg( GL.iPoints(iPt,:) );
			for i=1:2
				c_nOper{i} = sparse(nOper_eg(:,:,i));
			end
			% saving file:
			save(szFile,'c_nOper');
		end
	end
end
% Construct H_nonint matrices with block-diagonal values
function Set_H_nonint()
    % ACCESS:
    global GL
	
    nk = size(GL.kPoints,1);	
    GL.H_nonint_eg = zeros(16*nk,16*nk,2);    
    for i=1:nk
        ii = (i-1)*16+1;
        GL.H_nonint_eg(ii:ii+15,ii:ii+15,:) = Oper_Hnonint_eg( GL.kPoints(i,:) );
    end
end
% Construct H_1s3d matrices with core hole
function Set_H_corehole()
    % ACCESS:
    global GL  Eg
	
	% Core-Hole site:
	switch GL.CH
		case 0
			return;
		case 1
			site = [0 0];
		case 2
			site = [1 0];
		case 3
			site = [1 1];
		case 4
			if GL.N == 4
				site = [-1 0];	% this is due to iPoints choice
			else
				site = [3 0];
			end
		case 5
			site = [1 -1];
		case 6
			site = [2 -1];
		case 7
			site = [2 0];
		case 8
			site = [2 1];
		otherwise
			error('ERR: Erroneous CH value! CH can only be {0,1,..,8}');
	end
	GL.Track_iPoint = find(GL.iPoints(:,1)==site(1) & GL.iPoints(:,2)==site(2));
	if isempty(GL.Track_iPoint); error('ERR: Suggested tracking (CH) site is not within the domain of iPoints.');end;
    site = GL.iPoints(GL.Track_iPoint,:);
	
	% Eg CASE:
	GL.H_corehole_eg = Eg.C.Uch * sum( Oper_n_eg(site), 3 );
	GL.H_corehole_eg(:,:,2) = GL.H_corehole_eg(:,:,1);
end


% Update nXpt table
function Update_nXpt()
	% ACCESS:
	global GL
	
	% extract eigen vectors
	[vEg_up, eEg_up] = eig(GL.H_tot_eg(:,:,1));		eEg_up = diag(eEg_up);
	[vEg_dn, eEg_dn] = eig(GL.H_tot_eg(:,:,2));		eEg_dn = diag(eEg_dn);
	nnEg = size(vEg_up,1) / 8;
	
	% count number of spins/orbitals for all occupied electrons
	ii = [1 1];
	while sum(ii)-2 < 2*nnEg
		[~,ind] = min([ eEg_up(ii(1),1), eEg_dn(ii(2),1) ]);
		ii(ind) = ii(ind) + 1;
	end
	nEg_up = ii(1) - 1;
	nEg_dn = ii(2) - 1;

%{-
% Make spins even
nEg_up = nnEg;
nEg_dn = nnEg;
%}
	
	% just to keep an eye on the count
	if nEg_up ~= nnEg || nEg_dn ~= nnEg
		fprintf('!!! Split for %i electrons: [UP:eg|DN:eg]=[%i|%i]\n',...
			2*nnEg, nEg_up, nEg_dn);
	end
	
	LL1_up = vEg_up(:,1:nEg_up);
	LL1_dn = vEg_dn(:,1:nEg_dn);
	
	nPt = size(GL.iPoints,1);
	nXpt_list_new = zeros(nPt,4);
	c_nOper = cell(1,2);
	for iPt = 1:nPt
		load(sprintf(GL.file_nOper, iPt));	% load nOper into: c_nOper
		%{-
		n_xpt = [ ...
			sum( sum( (LL1_up' * c_nOper{1}).' .* LL1_up, 1)), ...
			sum( sum( (LL1_up' * c_nOper{2}).' .* LL1_up, 1)), ...
			sum( sum( (LL1_dn' * c_nOper{1}).' .* LL1_dn, 1)), ...
			sum( sum( (LL1_dn' * c_nOper{2}).' .* LL1_dn, 1)) ];
		%}
		%{
		n_xpt = zeros(1,4);
		% Eg spin UP case:
		for iv = 1:nEg_up
			L = vEg_up(:,iv);
			n_xpt(1:2) = n_xpt(1:2) + [L' * c_nOper{1} * L, L' * c_nOper{2} * L];
		end
		% Eg spin DOWN case:
		for iv = 1:nEg_dn
            L = vEg_dn(:,iv);
			n_xpt(3:4) = n_xpt(3:4) + [L' * c_nOper{1} * L, L' * c_nOper{2} * L];
		end
		%}
		nXpt_list_new(iPt,:) = real(n_xpt);
	end

	% Pullay Mixing segment:
	if GL.isPullay == 1
		if GL.Track.Size(1) >= 2
			alpha = GL.alphaPullay;
			nXpt_list_new = alpha		.* nXpt_list_new + ...
							(1-alpha)	.* GL.nXpt_list_old;
			GL.nXpt_list_old = GL.nXpt_list;
		else
			GL.nXpt_list_old = GL.nXpt_list;
		end
	elseif GL.isPullay == 2
		nXpt_list_new = GL.alphaPullay	.* nXpt_list_new + ...
						GL.betaPullay	.* GL.nXpt_list_old + ...
						(1-GL.alphaPullay-GL.betaPullay)	.* GL.nXpt_list_old2;
		GL.nXpt_list_old2 = GL.nXpt_list_old;
		GL.nXpt_list_old = GL.nXpt_list;
	end
	GL.nXpt_list = nXpt_list_new;
	
	% this is for data tracking and stopping criteria
	Eocc = sum(eEg_up(1:nEg_up)) + sum(eEg_dn(1:nEg_dn));
	GL.Track.Add([Eocc, nXpt_list_new(GL.Track_iPoint,:)], 1:5);
end
% Calculate total Hamiltonian
function Update_Htot()
    % ACCESS:
    global GL  Eg
    
	% Create U-based list for Hartree-Fock
	U_list = zeros(size(GL.nXpt_list));
	nXpt_sum = Eg.C.U * sum(GL.nXpt_list(:,1:4),2);
	for ic = 1:4
		U_list(:,ic) = nXpt_sum - Eg.C.U * GL.nXpt_list(:,ic);
	end
	
	% Create H_coulomb due to Hartree-Fock
	nPt = size(GL.iPoints,1);
	H_coulomb_eg = zeros(2*nPt,2*nPt, 2);
	c_nOper = cell(1,2);
	for iPt = 1:nPt
		load(sprintf(GL.file_nOper, iPt));	% load nOper into: c_nOper
		U = U_list(iPt,:);
		H_coulomb_eg(:,:,1) = H_coulomb_eg(:,:,1) + U(1)*c_nOper{1} + U(2)*c_nOper{2};
		H_coulomb_eg(:,:,2) = H_coulomb_eg(:,:,2) + U(3)*c_nOper{1} + U(4)*c_nOper{2};
	end

	GL.H_tot_eg = GL.H_nonint_eg + H_coulomb_eg;
	if GL.CH ~= 0
		GL.H_tot_eg = GL.H_tot_eg + GL.H_corehole_eg;
	end
end
function bval = isStopping()
	global GL
	
	choice = 1;	% {1,2}

	bval = false;
	switch choice
		case 1
			del_Eocc = abs(GL.Track.GetLast(1) - GL.Track.GetLast(1,-1));
			if del_Eocc < 1e-4
				bval = true;
			end
		case 2
			del_n = max( abs(GL.Track.GetLast(2:5) - GL.Track.GetLast(2:5,-1)) );
			if del_n < 1e-3
				bval = true;
			end
	end
end
% Converging Loop
function ConvergeData(maxSteps, bReloadLastState)
	global GL
	
	% set initial Hamiltonians
	if bReloadLastState == 0		
		GL.H_tot_eg = GL.H_nonint_eg;
	end
	fprintf('%sCALCULATING nXpt and H_tot^(eg): ...\n', StageId());
	tic;
	for step = 1:maxSteps
		fprintf('    * Step %03i/%03i\n', step,maxSteps);
		Update_nXpt();
		Update_Htot();
		% stopping criteria
		if isStopping(); break; end;
		% incremental saving for large calculations
		if GL.N >= 16; SaveLastState(); end;
	end
	toc;
end
%==========================================================
% PRINTING / SAVING:
function SaveLastState()
	% ACCESS to save
	global GL  Eg  stage_id
	
	% correct for ellapsed time
	GL.timediff = GL.timediff + cputime - GL.timecpu;
	GL.timecpu = cputime;
	
	save(sprintf('%s__LastState.mat',GL.file_main), ...
		'GL','Eg', 'stage_id');
end
function LoadLastState(N,CH)
	% ACCESS to overwrite
	global GL  Eg  stage_id
	
	szFile = sprintf('_N%02d_data_/_main_/N%02i_CH%i__LastState.mat',N,N,CH);
	if exist(szFile, 'file') == 2
		GL = {};
		Eg = {};
		Etot = {};
		ZnXpt = {};
		stage_id = {};
		load(szFile);
	else
		error('ERR: File [%s] does not exist. Set bReloadLastState=0 to generate the file.',szFile);
	end
	
	% correct for ellapsed time
	GL.timecpu = cputime;
end

function szId = StageId()
	global stage_id   % local static storage
	if isempty(stage_id); stage_id = 0; end;
	stage_id = stage_id + 1;
	szId = sprintf('## % 3i ## - ',stage_id);
end
function Table_nXpt()
	global GL

	cType = {'Mn3x','Mn4 ','Mn3y'};
	net_spin = GL.nXpt_list * [1 1 -1 -1]';
	
	data = [GL.iPoints(:,1:3), GL.nXpt_list(:,1:4), net_spin, sum(GL.nXpt_list(:,1:4), 2)];
	
	file = sprintf('%s__nXptTable.txt',GL.file_tables);
	fid = fopen(file, 'w');
	fprintf(fid, '+-----------------------------------------------------------+\r\n');
	fprintf(fid, '| N = %2i: ELECTRON NUMBER TABLE (CH = %i)                    |\r\n',GL.N,GL.CH);
	fprintf(fid, '+------+------+-----------------------------+--------+------+\r\n');
    fprintf(fid, ' i-pos | orb. |     eg case: < n_{..} >     |  net   | sum(n)\r\n');
    fprintf(fid, ' ix iy | type | -,up   +,up   -,dn   +,dn   |  spin  |       \r\n');
    fprintf(fid, '-------+------+-----------------------------+--------+-------\r\n');
	for i = 1:size(data,1)
		fprintf(fid, ' % i % i | %s | %6.4f %6.4f %6.4f %6.4f | %+5.3f | %6.4f\r\n', ...
			data(i,1:2), cType{data(i,3)}, data(i,4:9)  );
	end
	fprintf(fid, '+------+------+-----------------------------+--------+------+\r\n');
    fprintf(fid, '| CHECK: Sum(n)/N^2 = %5.3f                                 |\r\n', sum(sum(GL.nXpt_list))/size(GL.iPoints,1));
	fprintf(fid, '+-----------------------------------------------------------+\r\n');
	fclose(fid);
	
	fprintf('    * File [%s] have been generated.\n',file);
end
function Table_SysParams()
	global GL  Eg
	
	nxpt = [GL.nXpt_list(GL.iPoints(:,1)==0 & GL.iPoints(:,2)==0,:); ...
		GL.nXpt_list(GL.iPoints(:,1)==1 & GL.iPoints(:,2)==0,:); ...
		GL.nXpt_list(GL.iPoints(:,1)==1 & GL.iPoints(:,2)==1,:); ...
		GL.nXpt_list(GL.iPoints(:,1)==-1 & GL.iPoints(:,2)==0,:) ...
		];
	cTF = {'false','true '};
	cCH = {'n/a     ',...
		   'Mn3x+ up','Mn4+  up','Mn3y+ up','Mn4+  up',...
		   'Mn3x+ dn','Mn4+  dn','Mn3y+ dn','Mn4+  dn',};
	
	file = sprintf('%s__SysParams.txt',GL.file_tables);
	fid = fopen(file, 'w');
	fprintf(fid, '+----------------------------------------------+\r\n');
	fprintf(fid, '|        SIMULATION SYSTEM PARAMETERS          |\r\n');
	fprintf(fid, '+------------+---------------------------------+\r\n');
	fprintf(fid, '| System:    | N = %2i, Ni = %03i, Nk = %03i      |\r\n', GL.N, (GL.N^2), (GL.N^2/8));
	fprintf(fid, '+------------+---------+---------+-------------+\r\n');
	fprintf(fid, '| Parameters | Eg      | T2g     | Units       |\r\n');
	fprintf(fid, '+------------+---------+---------+-------------+\r\n');
	fprintf(fid, '| Q_1s       | % 6.4f |    -    | angstrom    |\r\n', Eg.C.Q_1s);
	fprintf(fid, '| Q_2s       | % 6.4f |    -    | angstrom    |\r\n', Eg.C.Q_2s);
	fprintf(fid, '| Q_3u       | % 6.4f |    -    | angstrom    |\r\n', Eg.C.Q_3u);
	fprintf(fid, '| Q_3s       | % 6.4f |    -    | angstrom    |\r\n', Eg.C.Q_3s);
	fprintf(fid, '+------------+---------+---------+-------------+\r\n');
	fprintf(fid, '| t0         | % 6.4f |    -    | eV          |\r\n', Eg.C.t0);
	fprintf(fid, '| beta       | % 6.4f |    -    |             |\r\n', Eg.C.beta);
	fprintf(fid, '| lambda     | % 6.4f |    -    | eV/angstrom |\r\n', Eg.C.lambda);
	fprintf(fid, '| JhSc       | % 6.4f |    -    | eV          |\r\n', Eg.C.JhSc);
	fprintf(fid, '| U          | % 6.4f |    -    | eV          |\r\n', Eg.C.U);
	fprintf(fid, '| Uch        | % 6.4f |    -    | eV          |\r\n', Eg.C.Uch);
	fprintf(fid, '| shift      | % 6.4f |    -    | eV          |\r\n', Eg.C.shift);
	fprintf(fid, '+------------+---------+---------+-------------+\r\n');
	Q_33 = (Eg.C.Q_3u + Eg.C.Q_3s);
	Q_34 = (Eg.C.Q_3u - Eg.C.Q_3s);
	Gap_br = 2 * Eg.C.lambda * Eg.C.beta * Eg.C.Q_1s;
	Gap_jt = Eg.C.lambda * (sqrt(4 * Eg.C.Q_2s^2 + Q_33^2) - abs(Q_34));
	Gap_tot = Gap_br + Gap_jt;
	Gap_jt_Mn3 = 2 * Eg.C.lambda * (sqrt(4 * Eg.C.Q_2s^2 + Q_33^2));
	fprintf(fid, '| Gap: br    | % 6.4f |    -    | eV          |\r\n', Gap_br);
	fprintf(fid, '| Gap: JT    | % 6.4f |    -    | eV          |\r\n', Gap_jt);
	fprintf(fid, '| Gap: Tot   | % 6.4f |    -    | eV          |\r\n', Gap_tot);
	fprintf(fid, '| Gap: JT Mn3| % 6.4f |    -    | eV          |\r\n', Gap_jt_Mn3);
	fprintf(fid, '+------------+---------+---------+-------------+\r\n');
	fprintf(fid, '| Select Eg n_xpt Values:                      |\r\n');
	fprintf(fid, '| ix iy :   -,up    +,up    -,dn    +,dn       |\r\n');
	fprintf(fid, '|  0  0 : %6.4f  %6.4f  %6.4f  %6.4f       |\r\n', nxpt(1,1:4));
	fprintf(fid, '|  1  0 : %6.4f  %6.4f  %6.4f  %6.4f       |\r\n', nxpt(2,1:4));
	fprintf(fid, '|  1  1 : %6.4f  %6.4f  %6.4f  %6.4f       |\r\n', nxpt(3,1:4));
	fprintf(fid, '| -1  0 : %6.4f  %6.4f  %6.4f  %6.4f       |\r\n', nxpt(4,1:4));
	fprintf(fid, '+------------+---------------------------------+\r\n');
	fprintf(fid, '| C-H Type   | %s                        |\r\n', cCH{GL.CH+1});
	if GL.CH == 0
		szCH = 'n/a    ';
	else
		szCH = sprintf('(% i,% i)',GL.iPoints(GL.Track_iPoint,1:2));
	end
	fprintf(fid, '| C-H Site   | %s                         |\r\n', szCH);
	fprintf(fid, '| isPullay   | %s                           |\r\n', cTF{logical(GL.isPullay)+1} );
	if GL.isPullay > 0
		fprintf(fid, '| alpha Pull.| %3.2f                            |\r\n', GL.alphaPullay);
	end
	fprintf(fid, '| Calc. Time | %s|\r\n', strjust(sprintf('%27.3f sec.',GL.timediff), 'left') );
	fprintf(fid, '| Cycles     | %s|\r\n', strjust(sprintf('%32i',GL.Track.Size()), 'left') );
	fprintf(fid, '| Time Stamp | %s            |\r\n', datestr(now));
	fprintf(fid, '+------------+---------------------------------+\r\n');
	fclose(fid);
	
	fprintf('    * File [%s] have been generated.\n',file);
end
%==========================================================
% FIGURE PLOTTING:
% private function
function Plot_Energies()
	% ACCESS:
	global GL  Eg
	
	E1 = sortrows([eig(GL.H_nonint_eg(:,:,1)); eig(GL.H_nonint_eg(:,:,2))]);
	E2 = sortrows([eig(GL.H_tot_eg(:,:,1)); eig(GL.H_tot_eg(:,:,2))]);
	ind = 1:size(E1,1);
	
	szCH = '';
	if GL.CH ~= 0
		szCH = sprintf(', U_{CH}=%4.1f_{eg} (eV)', Eg.C.Uch);
	end
	szTitle = sprintf('N=%i, CH=%i: Energy Profile for H_{eg}%s',GL.N,GL.CH,szCH);
	GL.figId = FigId(GL.figId, szTitle);
	hold on;
	plot(ind,E1, '.','MarkerSize',5, 'Color',[.2 .8 1]);
	plot(ind,E2, '.','MarkerSize',5, 'Color',[0 0 1]);

	minNN = min( E1 );
	maxNN = max( E2 );
	title(szTitle);
	ylabel('Energy - E_F (eV)');
	legend('E_{nonint}^{eg}' ,'E_{tot}^{eg}', 'Location','SouthEast');
	
	
	NN = size(E1,1);
	plot([1 NN], [0 0], ':k');
	fermi = size(E1,1) / 8 + 0.5;
	plot([fermi fermi], [minNN maxNN], ':k');
	text(fermi,maxNN-.1,'occupied', ...
		'Rotation',90, 'HorizontalAlignment','right', 'VerticalAlignment','bottom' );
	text(fermi,maxNN-.1,'unoccupied', ...
		'Rotation',90, 'HorizontalAlignment','right', 'VerticalAlignment','top' );
	hold off;
end
function Plot_Tracking()
	global GL  Eg
	
	data = GL.Track.GetData();
	n = size(data,1);
	
	Eocc = data(:,1);
	Sp_up = data(:,[2 3]);
	Sp_dn = data(:,[4 5]);
	clrE = [0 .6 0];
	clrEg = [0 0 1];
	
	GL.figId = FigId(GL.figId, 'Evolution of various parametes.');
	
	subplot(3,1,1);
    plot(Eocc,'.-', 'Color',clrE);
	szCH = '';
	if GL.CH == 1
		szCH = sprintf(', U_{CH}=%4.1f_{eg} (eV)', Eg.C.Uch);
	end
	title(sprintf('N=%i, CH=%i: E_{tot}^{OCC} Tracking, E = %.2f (eV)%s', GL.N,GL.CH, Eocc(n), szCH ) );
    xlabel('Steps');
    ylabel('Energy of Occupied Sites (eV)');
	xlim([1,n]);
	ylim([min(Eocc)-5, max(Eocc)+5]);
	
	szSite = sprintf('(%i,%i)', GL.iPoints(GL.Track_iPoint,1:2));
	
	subplot(3,1,2); hold on;
	plot(Sp_up(:,1),'.-', 'Color',clrEg);
	plot(Sp_up(:,2),'.--', 'Color',clrEg);
	hold off;
	title(sprintf('Spin \\uparrow n-Values at site %s',szSite));
	xlabel('Steps');
	xlim([1,n]);
	ylim([-0.1, 1.1]);
	legend(sprintf('n^{eg}_{-,\\uparrow} = %5.3f',Sp_up(n,1)),...
		sprintf('n^{eg}_{+,\\uparrow} = %5.3f',Sp_up(n,2)),...
		'Location','NorthWest');
	
	subplot(3,1,3); hold on;
	plot(Sp_dn(:,1),'.-', 'Color',clrEg);
	plot(Sp_dn(:,2),'.--', 'Color',clrEg);
	hold off;
	title(sprintf('Spin \\downarrow n-Values at site %s',szSite));
	xlabel('Steps');
	xlim([1,n]);
	ylim([-0.1, 1.1]);
	legend(sprintf('n^{eg}_{-,\\downarrow} = %5.3f',Sp_dn(n,1)),...
		sprintf('n^{eg}_{+,\\downarrow} = %5.3f',Sp_dn(n,2)),...
		'Location','NorthWest');
end

function Plot_ColoredAtoms(data_C, minmax_C, szTitle)
	global GL
	
	MS = 40;	% marker size limits
	nPt = size(GL.iPoints,1);
	
	% define orbital loops
	orb_size = 0.5;
	th = 0:0.1:2*pi+0.1;
	r_hor = orb_size * cos(th).^2;
	r_ver = orb_size * sin(th).^2;
	xy_hor = [r_hor.*cos(th); r_hor.*sin(th)];
	xy_ver = [r_ver.*cos(th); r_ver.*sin(th)];
	clrCH =  [1 .9 .9];
	clrOrb = [1 1 1]*.8;
	clrZig = [1 1 1]*.95;

	GL.figId = FigId(GL.figId, szTitle);
	
	hold on;
	% draw CH site
	if GL.CH ~= 0
		CH_site = GL.iPoints(GL.Track_iPoint,:);
		plot(CH_site(1),CH_site(2),'.', 'Color',clrCH, 'MarkerSize',MS*4);
	end
	% draw zig-zag and orbitals
	for iPt = 1:nPt
		x = GL.iPoints(iPt,1);
		y = GL.iPoints(iPt,2);
		switch GL.iPoints(iPt,3)
			case 1
				plot(x+[-1 1],y+[0 0],'-', 'Color',clrZig,'LineWidth',3);
				plot(x+xy_hor(1,:),y+xy_hor(2,:),'-', 'Color',clrOrb);
			case 3
				plot(x+[0 0],y+[-1 1],'-', 'Color',clrZig,'LineWidth',3);
				plot(GL.iPoints(iPt,1)+xy_ver(1,:),GL.iPoints(iPt,2)+xy_ver(2,:),'-', 'Color',clrOrb);
		end
	end
	% draw color dots
	nC = 64;
	cmap = colormap( jet(nC) );
	for iPt = 1:nPt
		q = (data_C(iPt) - minmax_C(1)) ./ (minmax_C(2) - minmax_C(1));
		iColor = max( 1, min(nC, round((nC-1)*q+1) ) );
		plot(GL.iPoints(iPt,1),GL.iPoints(iPt,2),'.', 'MarkerSize',MS, 'Color',cmap(iColor,:));
	end

	hold off;
	colorbar;
	caxis(minmax_C);
	minmax_xy = [min(GL.iPoints(:,1)), max(GL.iPoints(:,1))];
	xlim(minmax_xy + [-.6, .6]);
	ylim(minmax_xy + [-.6, .6]);
	title(szTitle);
	xlabel('i_x');
	ylabel('i_y');
	set(gca,'XTick',(minmax_xy(1):minmax_xy(2)), 'YTick',(minmax_xy(1):minmax_xy(2)), ...
		'DataAspectRatio',[1 1 1]);
end
function Plot_NetSpin()
	global GL
	
	net_spin = GL.nXpt_list * [1 1 -1 -1]';
	msp = max(abs(net_spin));
	minmax_C = [-msp,+msp];	
	Plot_ColoredAtoms(net_spin, minmax_C, ...
		sprintf('N=%i, CH=%i: Atomic Net Spin',GL.N,GL.CH));
end
function Plot_NetCharge()
	global GL
	
	net_charge = sum(GL.nXpt_list(:,1:4), 2);
	minmax_C = [0,max(net_charge)];
	Plot_ColoredAtoms(net_charge, minmax_C, ...
		sprintf('N=%i, CH=%i: Atomic Net Charge',GL.N,GL.CH));
end
%==========================================================
% EXPORT DATA FOR FURTHER ANALYSIS:
function Save_nDiff()
	global GL  Eg
	
	data.nXpt_list = GL.nXpt_list;
data.U_eg = Eg.C.U;
data.Uch_eg = Eg.C.Uch;
	data.C = Eg.C;	
	if GL.CH == 0
		% data file without Core Hole, used as a basis
		data.N = GL.N;
		data.figId = 20;
		data.iPoints = GL.iPoints;
	else
		data.CH_site = GL.iPoints(GL.Track_iPoint,:);
	end
	eval( sprintf('CH%d = data;',GL.CH) );
	save(GL.file_ndiff, sprintf('CH%d',GL.CH));
end
function Save_RIXS()
	global GL  Eg

 	data.H_eg = GL.H_tot_eg;
	data.N = GL.N;
	data.CH = GL.CH;
	data.C = Eg.C;
	if GL.CH == 0
		% data without Core Hole, used as a basis		
		data.kPoints = GL.kPoints;
		CH0 = data;	%#ok
		save(GL.file_rixs, 'CH0');
	else
		% data with Core Hole
		data.CH_site = GL.iPoints(GL.Track_iPoint, 1:2);
		CH1 = data;	%#ok
		save(GL.file_rixs, 'CH1');
	end	
end

function Save_Hcoloumb_Bands()
	global GL

	% n-values are only valid for CH=0, for band's H_coulomb
	if GL.CH ~= 0; return; end;
	file = sprintf('%s__nXpt_list.mat', GL.file_main);
	
	iPoints = GL.iPoints;
	nXpt_list = GL.nXpt_list;
	save(file, 'iPoints','nXpt_list');
end
%==========================================================

function ExtractDataForUch()
	global GL  Eg
	
	fid = fopen(sprintf('DataForUch_N%02d_t%.2f.txt', GL.N, Eg.C.t0), 'a');
	
	ch = [0,0; 1,0; 1,1; -1,0];
	if GL.CH == 0
		for i=1:2%4
			indCH = find(GL.iPoints(:,1)==ch(i,1) & GL.iPoints(:,2)==ch(i,2));
			S = sum(GL.nXpt_list(indCH,:));
			fprintf(fid, '%d  % d  % d  %.4f  %.4f\r\n', ...
				GL.CH, ch(i,1),ch(i,2), Eg.C.Uch, S);
		end
	else
			i = GL.CH;
			indCH = find(GL.iPoints(:,1)==ch(i,1) & GL.iPoints(:,2)==ch(i,2));
			S = sum(GL.nXpt_list(indCH,:));
			fprintf(fid, '%d  % d  % d  %.4f  %.4f\r\n', ...
				GL.CH, ch(i,1),ch(i,2), Eg.C.Uch, S);
	end
	fclose(fid);
end

function GetEocc()
	global GL
	
	Eup = eig(GL.H_tot_eg(:,:,1));
	Edn = eig(GL.H_tot_eg(:,:,2));
	
	nF = size(Eup,1) / 8;
	Eocc = [Eup(1:nF), Edn(1:nF)];
	Eocc = sum(sum(Eocc));
	Eocc
end

























