% Entry point (Hit F5 to run)
%{
===============================================================================
DESIGNED BY:    Tsezar F. Seman
AFFILIATION:    NJIT University, Physics Department
MODIFIED ON:    06/06/2012
COPYRIGHT:      (c) Tsezar F. Seman. All rights reserved.
PURPOSE:        Calculating RIXS data for eg case.
===============================================================================
%}
function run2_rixs_calc(N,minmax_E)
	clc; addpath('_functions_','_common_');
	if nargin < 1
		clear all;
		N = 8;		% must be divisible by 4
	end;
	if nargin < 2
		% energy bounds for RIXS table
		minmax_E = [-1, 20];	% units: eV
	end
	
	%--------------------------------------------
	% GENERATE RIXS DATASET:
	CmdTitle(sprintf('LaSrMnO4 RIXS: N = %i', N));
	fprintf('## SETTING GLOBAL PARAMS... \n');
	tic; SetGlobals(N); toc;
	fprintf('\n## CALCULATING RIXS... \n');
	tic; SetRIXS(minmax_E); toc;
	fprintf('\n## SAVING CALCULATED RIXS: ... \n');
	SaveCalcRixs();

	%--------------------------------------------
	% SAVE RESULTS AS TABLES:
	fprintf('\n## SAVING DATA TABLES: ... \n');
	TableEnergy();
	TableIntensity();
	TableSumIntensity();
	
	Table_Matrix_kMap();
	%--------------------------------------------
	fprintf('\nFINISHED... \n');
end
%==========================================================
% PREPARE DATA FROM CH CORE-HOLE SITES:
% Collect all the extended points for the reduced Bruilloin zone and
% their indexes mapped to the kPoints
function [kPoints_ext, ind_ext] = GetExtended_kPoints(kPoints)
	sq = 1/sqrt(2);
	ROT = [sq sq; -sq sq];	% +45 rotation
	kR = kPoints * ROT;
	
	del = 1e-10;
	v1 = pi / 2 / sqrt(2);
	v2 = pi / sqrt(2);

	% stage 1: x
	ind1 = find(abs(kR(:,1)-v1) < del);
	kR_ext1 = kR(ind1,:);
	kR_ext1(:,1) = kR_ext1(:,1) - v2;
	% stage 2: y
	ind2 = find(abs(kR(:,2)-v1) < del);
	kR_ext2 = kR(ind2,:);
	kR_ext2(:,2) = kR_ext2(:,2) - v2;
	% stage 3: x & y
	ind3 = find(abs(kR(:,1)-v1) < del & abs(kR(:,2)-v1) < del);
	kR_ext3 = kR(ind3,:);
	kR_ext3(:,1:2) = kR_ext3(:,1:2) - v2;

	% finilize
	kR_ext = [kR_ext1, ind1; kR_ext2, ind2; kR_ext3, ind3];
	kPoints_ext = kR_ext(:,1:2) * ROT';
	ind_ext = kR_ext(:,3);
end
function [kPoints, kPointsExt, kIndexExt] = GetFull_kPoints(N)
	dk = 2*pi/N;
	kPoints = zeros(N^2, 2);
	kx = dk*(-N/2+1:N/2);
	ky = kx;
	nn = length(kx);
	for iy = 1:nn
		i = (iy-1)*nn+1;
		kPoints(i:i+nn-1,1) = kx';
		kPoints(i:i+nn-1,2) = ky(iy);
	end
	
	% extended k-points
	kPointsExt = [-pi, -pi];
	kIndexExt = N^2;
	
	kSet = [kx', 0.*kx']; kSet(:,2) = -pi;
	kPointsExt = [kPointsExt; kSet];
	kIndexExt = [kIndexExt; (N^2-N+1:N^2)'];
	
	kSet = [0.*kx', kx']; kSet(:,1) = -pi;
	kPointsExt = [kPointsExt; kSet];
	kIndexExt = [kIndexExt; (N:N:N^2)'];
end
function [V_CH0,MAP_CH0] = GetBaseCH0(H_CH0)
	global GL
	
	% allocate space
	nk = size(GL.kPoints,1);
	nE = size(H_CH0,1);			% # of electorns, total (per spin)
	V_CH0    = zeros(nE,nE, 2); % storage for column eigen-vectors
	MAP_CH0 = zeros(nE, 6, 2);	% storage for [E,kPoint_index,band_index,  E-H index,kx,ky]
	iBands = (1:16)';
	% go through every k-point
	jj = 1:16;	
	for ik = 1:nk
		% k-point
		k = GL.kPoints(ik,:);
		% spin up/down
		for spin = 1:2
			[V,E] = eig( H_CH0(jj,jj,spin) ); E = diag(E);
			V_CH0(jj,jj,spin)  = V;			% eigen vectors
			MAP_CH0(jj,1,spin) = E;			% energies
			MAP_CH0(jj,2,spin) = ik;		% kPoint index
			MAP_CH0(jj,3,spin) = iBands;	% band index
			MAP_CH0(jj,5,spin) = k(1);		% k_x
			MAP_CH0(jj,6,spin) = k(2);		% k_y
		end
		% iteration
		jj = jj + 16;
	end	
	% sort everything with increasing energy
	for spin = 1:2
		MAP_CH0(:,1,spin) = ReNormalize(MAP_CH0(:,1,spin));	
		[MAP_CH0(:,:,spin), ind] = sortrows(MAP_CH0(:,:,spin), 1);
		V_CH0(:,:,spin) = V_CH0(:,ind,spin);
	end
	% set E-H index, used in RIXS table
	MAP_CH0(:,4, 1) = 1:2:2*nE-1;
	MAP_CH0(:,4, 2) = 2:2:2*nE;
end
function [Beta_occ, CoF, Beta_unc, Z, CH_iPoint, Eocc] = SetBetaMatrices(inv_V_CH0,MAP_CH0, file_CH, CH)
	% load calculated core-hole Hamiltonian into CH1:
	load( file_CH );
	if CH1.CH ~= CH
		disp('ERR: Core hole input data file mismatch.');
	end
	[V_up,E_up] = eig(CH1.H_eg(:,:,1));
	[V_dn,E_dn] = eig(CH1.H_eg(:,:,2));
	BetaFull_up = inv_V_CH0(:,:,1) * V_up;
	BetaFull_dn = inv_V_CH0(:,:,2) * V_dn;
	
	% allocate space
	nE = size(MAP_CH0,1);
	nF = nE / 8;	
	Beta_occ = zeros(   nF,nF,2);
	CoF      = zeros(   nF,nF,2);
	Beta_unc = zeros(nE-nF,nF,2);
	
	% extract return values
	Beta_occ(:,:,1) = BetaFull_up(1:nF,1:nF);
	Beta_occ(:,:,2) = BetaFull_dn(1:nF,1:nF);
	
	Z = [det(Beta_occ(:,:,1)), det(Beta_occ(:,:,2))];
	
	CoF(:,:,1) = ( Z(1) .* inv(Beta_occ(:,:,1)) ).';
	CoF(:,:,2) = ( Z(2) .* inv(Beta_occ(:,:,2)) ).';
	
	Beta_unc(:,:,1) = BetaFull_up(nF+1:nE,1:nF);
	Beta_unc(:,:,2) = BetaFull_dn(nF+1:nE,1:nF);
	
	CH_iPoint = CH1.CH_site;
	
	% get E_occ, the energy of occupied states for given C-H
	E_up = diag(E_up);
	E_dn = diag(E_dn);
	Eocc = sum(sum( [E_up(1:nF), E_dn(1:nF)] ));
end
function SetGlobals(N)
	% starting with blank GL and blank TMP:
	global GL
	
	if mod(N,4)~=0 || N<4
		error('ERR: N must be divisible by 4, and N >= 4.');
	end

	GL.path = sprintf('_N%02i_data_',N);
	
	% load calculated base Hamiltonian into CH0:
	load( sprintf('%s/_RIXS_/N%02i_CH0.mat', GL.path, N) );
	
	GL.N = N;
	GL.C = CH0.C;
	
% DELETE: redundant, left from older version:
%GL.U_eg = CH0.C.U;
%GL.Uch_eg = CH0.C.Uch;
	
% DELETE:
%GL.figId = 30;
	GL.dir_tables = [GL.path,'/_tables_'];
	GL.kPoints = CH0.kPoints;
% DELETE:
%GL.figPref = sprintf('N=%i',N);
	
	% extended kPoints and indexes connecting with GL.kPoints
	[kPoints_ext, ind_ext] = GetExtended_kPoints(GL.kPoints);
	GL.kPointsExt = kPoints_ext;
	GL.kIndexExt = ind_ext;
	
	% k-points for extended Brillouin zone
	[GL.kPointsFull, GL.kPointsFullExt, GL.kIndexFullExt] = GetFull_kPoints(N);

	% return data from base Hamiltonian (CH0) ordered in increasing energy
	[V_CH0,MAP_CH0] = GetBaseCH0(CH0.H_eg);
	GL.MAP_CH0 = MAP_CH0;
	
	% create inverse once for re-occuring multiplication
	inv_V_CH0 = zeros(size(V_CH0));
	inv_V_CH0(:,:,1) = inv(V_CH0(:,:,1).').';
	inv_V_CH0(:,:,2) = inv(V_CH0(:,:,2).').';

%{
TFS NOTES:
	TMP.Beta_occ is redundant since we need TMP.CoF,
	but keep for testing, for now!
%}
	
	% prepare beta matrices
	nCH = 8;
	nE = size(CH0.H_eg,1);
	nF = nE / 8;								% 8 -> electron filling
	GL.Beta_occ = zeros(   nF, nF, 2, nCH);	% 2 -> spins
	GL.CoF      = zeros(   nF, nF, 2, nCH);	% 2 -> spins
	GL.Beta_unc = zeros(nE-nF, nF, 2, nCH);	% 2 -> spins
	GL.Z = zeros(nCH,2);						% 2 -> spins
	GL.CH_iPoint = zeros(nCH, 2);				% 2 -> ix,iy
	vEocc_CH = zeros(nCH,1);
	for CH = 1:nCH
		file_CH = sprintf('%s/_RIXS_/N%02i_CH%i.mat', GL.path,N,CH);
		[Beta_occ, Cof, Beta_unc, Z, CH_iPoint, E_occ] = SetBetaMatrices(inv_V_CH0,MAP_CH0, file_CH, CH);
		GL.Beta_occ(:,:,:, CH) = Beta_occ;
		GL.CoF(:,:,:, CH) = Cof;
		GL.Beta_unc(:,:,:, CH) = Beta_unc;
		GL.Z(CH,:) = Z;
		GL.CH_iPoint(CH,:) = CH_iPoint;
		vEocc_CH(CH,1) = E_occ;
	end
	
	% calculating resonant term for each CH case
	Eocc_CH0 = sum(sum( [GL.MAP_CH0(1:nF,1,1), GL.MAP_CH0(1:nF,1,2)] ));
	omega_in = (vEocc_CH(2)-Eocc_CH0 + vEocc_CH(1)-Eocc_CH0) / 2;
	g1 = GL.C.Gamma1;
	g2 = GL.C.Gamma2;
	vGamma = 1i * [g1,g2, g1,g2, g1,g2, g1,g2]';
	GL.RES = 1.0 ./ (omega_in - (vEocc_CH - Eocc_CH0) + vGamma);
	
	%{-
	% adding distortions at Mn4+ sites:
	ds = 0.01871;
	dd = [  0,0; ...
			 ds,-ds;...
			0,0; ...
			-ds, ds;...
			0,0; ...
			 ds,-ds;...
			0,0; ...
			-ds, ds ];
	GL.CH_iPoint = GL.CH_iPoint + dd;
	%}
end

function SetRIXS(minmax_E)
	global GL
	
	nE = size(GL.MAP_CH0,1);
	nF = nE / 8;
	nk = size(GL.kPoints, 1);
	nRows = 8*nk*nk * 14*2 * 2;		% same as: (nE-nF)*nF * 8 * 2
	iRow = 1;
	fprintf('    * Calculating RIXS table with %i rows...\n', nRows);
	
	% RixsData Columns:
	% 1-hole id, 2-electron id, 3-intensity, 4-energy
	% 5:6-momentum, 7-spin, 8-indexes of k-points
	RixsData = zeros(nRows,8);
	DK = [0,0;  pi,0;  0,pi;  pi,pi;  -pi/2,-pi/2;  pi/2,-pi/2;  -pi/2,pi/2;  pi/2,pi/2];
	mapOCC = GL.MAP_CH0(   1:nF,:, :);
	mapUNC = GL.MAP_CH0(nF+1:nE,:, :);
	rowUNC = zeros(8,nF, 2);
	% amplitude calculation terms, for every {ke,le,kh,lh,  spin,K}:
	PH = zeros(8,1);							% phase
	X1 = zeros(8,1);							% one CH excitation
	X0 = conj(GL.Z(:,1)) .* conj(GL.Z(:,2));	% no excitation
	RES = GL.RES;								% resonance denominator
	for iUNC = 1:(nE-nF)
		% extract rows for use with CoF
		for CH = 1:8
			rowUNC(CH,:, 1) = GL.Beta_unc(iUNC,:, 1,CH);
			rowUNC(CH,:, 2) = GL.Beta_unc(iUNC,:, 2,CH);
		end
		
		% populate RIXS table
		for iOCC = 1:nF
			for spin = 1:2
				% excitation contribution
				% example for spin up: Z_dn * DETERMINANT^[Beta(SwapedRow)_up]
				spin_neg = mod(spin,2)+1;
				for CH = 1:8
					X1(CH,1) = GL.Z(CH,spin_neg) .* (rowUNC(CH,:, spin) * (GL.CoF(iOCC,:, spin,CH)).');
				end
				
%{
% TEST DETERMINANT:
	CH = 1;
	r = GL.Beta_unc(iUNC,:, spin,CH);
	B = GL.Beta_occ(:,:, spin,CH);
	B(iOCC,:) = r;
	d1 = det(B);
	d2 = (rowUNC(CH,:, spin) * (GL.CoF(iOCC,:, spin,CH)).');
	err = abs(d1 - d2);
	if err > 1e-15
		err
	end
%}

				% k-indexes, band-indexes
				ik_h = mapOCC(iOCC,2, spin);	ik_e = mapUNC(iUNC,2, spin);
%				il_h = mapOCC(iOCC,3, spin);	il_e = mapUNC(iUNC,3, spin);
				iH   = mapOCC(iOCC,4, spin);	iE   = mapUNC(iUNC,4, spin);
				% momentum change in 1-st BZ
				dk = GL.kPoints(ik_e,:) - GL.kPoints(ik_h,:);
				% energy
				w = mapUNC(iUNC,1, spin) - mapOCC(iOCC,1, spin);
				
				for iDK = 1:8
					% momentum change in the extended BZ
					[q, iq] = CorrectQpt(-dk - DK(iDK,:));
					% phase
					PH = exp(-1i * (GL.CH_iPoint * q') );
					
					% amplitude for all CH
					%A = sum(PH .* X1 .* X0 .* RES,  1);
					A = sum(PH .* X1 .* X0,  1);
%A_tmp = PH .* X1 .* X0;
%A = A_tmp(3);
					
					% one data row for rixs table
					RixsRow = [
						iH, ...			% hole id
						iE, ...			% electron id
						A*conj(A), ...	% rixs intensity
						w, ...			% w energy
						q(1), ...		% q_x momentum
						q(2), ...		% q_y momentum
						spin, ...		% spin
						iq ...			% q-index
						];
					RixsData(iRow,:) = RixsRow;
					iRow = iRow+1;
				end  % iDK
			end  % spin
		end  % iOCC
	end  % iUNC
	if iRow-1 ~= nRows
		disp('ERR: RIXS Table - number of processed rows mismatched.');
	end
	
%	RixsData = RixsData(1:iRow-1,:);
	
	% reduce table to given energy bouds
	RixsData = RixsData(RixsData(:,4) <= minmax_E(2), :);
	RixsData = RixsData(RixsData(:,4) >= minmax_E(1), :);
		
	% set new k-points for extended BZ
	GL.kPoints = GL.kPointsFull;
	GL.kPointsExt = GL.kPointsFullExt;
	GL.kIndexExt = GL.kIndexFullExt;
	
%{-
	% Averaging w.r.t. qx <-> -qx:
	% create swap array for ik
	N = GL.N;
	nk = size(GL.kPoints,1);
	ind_swap = (1:nk)';
	for iOff = 0:N-1
		for i = 1:N-1
			ind_swap(iOff*N + i) = iOff*N + N-i;
		end
	end
	% process new table
	RixsData_qxNeg = RixsData;
	for i = 1:size(RixsData_qxNeg,1)
		ik = ind_swap( RixsData_qxNeg(i,8) );
		RixsData_qxNeg(i,8) = ik;
		RixsData_qxNeg(i,5) = GL.kPoints(ik,1);
	end
	% merge with existing table, and averaging intensity
	RixsData = [RixsData; RixsData_qxNeg];
	RixsData(:,3) = RixsData(:,3) / 2;
%}	
	
	% sort by descending intensity and pass to global
	GL.RIXS = sortrows(RixsData, -3);
	
	% integrate all intensities, separately per spin
	nk = size(GL.kPoints,1);	% must be recalculated, now for extended BZ
	SumIrixs = zeros(nk,2);
	for ik = 1:nk
		SumIrixs(ik,1) = sum( RixsData(RixsData(:,8) == ik & RixsData(:,7) == 1, 3) );
		SumIrixs(ik,2) = sum( RixsData(RixsData(:,8) == ik & RixsData(:,7) == 2, 3) );
	end
	if abs(sum(sum(SumIrixs)) - sum(RixsData(:,3))) > 1e-12
		disp('ERR: Sum Irixs mismatch. See SetRIXS().');
	end
	GL.RIXS_SUM = SumIrixs;
	
	% cleanup GL
	GL = rmfield(GL, 'kPointsFull');
	GL = rmfield(GL, 'kPointsFullExt');
	GL = rmfield(GL, 'kIndexFullExt');
	GL = rmfield(GL, 'Beta_occ');
	GL = rmfield(GL, 'CoF');
	GL = rmfield(GL, 'Beta_unc');
	GL = rmfield(GL, 'Z');
	GL = rmfield(GL, 'CH_iPoint');
end
%==========================================================
% This is  necessary to eliminate 14/15-th significant
% digit discrepancy in degenerate values
function EE = ReNormalize(E)
	[EE, ind] = sortrows(E,1);
	dig_eps = 1000*eps(EE(1));
	
	igr = 1;
	groups(igr,1) = 1;
	val = EE(1);
	for i = 2:size(EE,1)
		% if values are not close
		if abs(EE(i) - val) > dig_eps
			val = EE(i);
			groups(igr,2) = i-1;
			igr = igr+1;
			groups(igr,1) = i;
		end
	end
	groups(igr,2) = i;
	
	% fixing values
	for i = 1:size(groups,1)
		a = groups(i,1);
		b = groups(i,2);
		avg = mean( EE(a:b,1) );
		EE(a:b,1) = avg;
	end
	
	EE = sortrows([EE, ind],2);
	EE = EE(:,1);
end

% map new q-value onto extanded BZ
function [q, iq] = CorrectQpt(q)
	global GL
	
	Eps = 1e-10;
	% kx values
	if q(1) > pi
		q(1) = q(1) - 2*pi;
	elseif q(1) <= -pi + Eps
		q(1) = q(1) + 2*pi;
	end
	% ky values
	if q(2) > pi
		q(2) = q(2) - 2*pi;
	elseif q(2) <= -pi + Eps
		q(2) = q(2) + 2*pi;
	end
	% indexes
	kk = (GL.kPointsFull(:,1) - q(1)).^2 + (GL.kPointsFull(:,2) - q(2)).^2;
	iq = find(kk < 1e-10);	
	q = GL.kPointsFull(iq,:);
end
%==========================================================
% SAVING DATA:
function TableEnergy()
	global GL
	
	file = sprintf('%s/N%02i__RIXS_Energies.txt',GL.dir_tables,GL.N);
	fid = fopen(file, 'w');
	fprintf(fid, '+--------------------------------------------+\r\n');
	fprintf(fid, '| N = %2i: ELECTRON ENERGY SPECTRUM           |\r\n',GL.N);
	fprintf(fid, '------+-----------+-----------------+----+----\r\n');
	fprintf(fid, 'iE/iH |   Energy  |BZ:   kx      ky |band| sp \r\n');
	fprintf(fid, '------+-----------+-----------------+----+----\r\n');
	
	nT = size(GL.MAP_CH0,1);
	nF = nT/8;
	cSp = {'u ',' d'};
	for i = nT:-1:1
		% spin down/up
		for spin = [2,1]
			v = GL.MAP_CH0(i,:,spin);
			fprintf(fid, ' %4i | %9.4f |%s%s | %2d | %s\r\n', v(4), v(1), ...
				Num2Latex(v(5),0,[8,4]), Num2Latex(v(6),0,[8,4]), v(3), cSp{spin});
		end
		% fermi level separator
		if i == nF+1
			fprintf(fid, '>---------------< FERMI LEVEL >--------------<\r\n');
		end
	end
	fprintf(fid, '------+-----------+-----------------+----+----\r\n');
	fclose(fid);
	
	fprintf('    * File [%s] have been generated.\n',file);
end
function TableIntensity()
	global GL

	file = sprintf('%s/N%02i__RIXS_Intensity.txt',GL.dir_tables,GL.N);
	fid = fopen(file, 'w');
	fprintf(fid, '+----------------------------------------------------------------+\r\n');
	fprintf(fid, '| N = %2i: RIXS CALCULATION - PARTIAL TABLE                       |\r\n',GL.N);
	fprintf(fid, '------+------+--------------------+---------+-----------------+---\r\n');    
	fprintf(fid, ' H_id | E_id |       I_RIXS       |  del_w  |    del_q(x,y)   | sp\r\n');
	fprintf(fid, '------+------+--------------------+---------+-----------------+---\r\n');
	nR = min(size(GL.RIXS,1), 170); 	% limit to about 3 pages in PDF
	cSp = {'u ',' d'};
	for i = 1:nR
		fprintf(fid, ' %4i | %4i | %.16f |%8.4f |%s%s | %s\r\n', GL.RIXS(i,1:4), ...
			Num2Latex(GL.RIXS(i,5),0,[8,4]), Num2Latex(GL.RIXS(i,6),0,[8,4]), cSp{GL.RIXS(i,7)});
	end
	fprintf(fid, '  ... |  ... |                ... |     ... |     ...     ... | ..\r\n');
	fprintf(fid, '------+------+--------------------+---------+-----------------+---\r\n');
	fclose(fid);
	
	fprintf('    * File [%s] have been generated.\n',file);
end
function TableSumIntensity()
	global GL

	file = sprintf('%s/N%02i__RIXS_SumIntensity.txt',GL.dir_tables,GL.N);
	fid = fopen(file, 'w');
	fprintf(fid, '+--------------------------------------------------------------+\r\n');
	fprintf(fid, '| N = %2i: Integrated I_RIXS                                    |\r\n',GL.N);
	fprintf(fid, '-----+--------------------+--------------------+----------------\r\n');   
	fprintf(fid, 'k_id | Sum I_RIXS_up      | Sum I_RIXS_dn      |      kx      ky\r\n');
	fprintf(fid, '-----+--------------------+--------------------+----------------\r\n');
	Nk = size(GL.kPoints,1);
	for ik = 1:Nk
		fprintf(fid, '  %2i | %.16f | %.16f |%s%s\r\n', ik, GL.RIXS_SUM(ik,:), ...
			Num2Latex(GL.kPoints(ik,1),0,[8,4]), Num2Latex(GL.kPoints(ik,2),0,[8,4]) );
	end
	fprintf(fid, '-----+--------------------+--------------------+----------------\r\n');
	fclose(fid);
	
	fprintf('    * File [%s] have been generated.\n',file);
end
function Table_Matrix_kMap()
	global GL
	
	sPref = sprintf('%s/N%02i__kMap',GL.dir_tables,GL.N);
	
	% export coordinate layout
	kxy = GL.kPoints(1:GL.N,1);
	file = sprintf('%s_kx-ky.txt',sPref);
	fid = fopen(file, 'w');
	fprintf(fid, 'kx values:\r\n');
	fprintf(fid, '% .16f ',kxy);
	fprintf(fid, '\r\n\r\nky values:\r\n');
	fprintf(fid, '% .16f\r\n',flipud(kxy));
	fclose(fid);
	
	% export k-map matrix values
	nD = sum(GL.RIXS_SUM, 2);
	nD2 = flipud(reshape(nD, GL.N, GL.N)');
	file = sprintf('%s_matrix_rixs.txt',sPref);
	fid = fopen(file, 'w');
	for ir = 1:size(nD2,1)
		fprintf(fid, ' % .16f',nD2(ir,:));
		fprintf(fid, '\r\n');
	end
	fclose(fid);

	fprintf('    * Table file [%s] have been generated.\n',file);
end
%==========================================================
% SAVING/LOADING:
function SaveCalcRixs()
	% ACCESS to save
	global GL
	
	save(sprintf('%s/_RIXS_/N%02i_CalcRixs.mat',GL.path,GL.N), 'GL');
end
%==========================================================
