% Entry point (Hit F5 to run)
%{
===============================================================================
DESIGNED BY:    Tsezar F. Seman
AFFILIATION:    NJIT University, Physics Department
MODIFIED ON:    06/06/2012
COPYRIGHT:      (c) Tsezar F. Seman. All rights reserved.
PURPOSE:        Calculating RIXS data for eg case.
===============================================================================
%}
function run2_rixs_calc_ext(N,minmax_E)
	clc; addpath('_functions_','_common_');
	if nargin < 1
		clear all;
		N = 16;		% must be divisible by 4
	end;
	if nargin < 2
		% energy bounds for RIXS table
		minmax_E = [-1, 20];	% units: eV
	end
	
	%--------------------------------------------
	% GENERATE RIXS DATASET:
	CmdTitle(sprintf('LaSrMnO4 RIXS: N = %i', N));
	fprintf('## SETTING GLOBAL PARAMS... \n');
	tic; SetGlobals(N); toc;
	fprintf('\n## CALCULATING RIXS... \n');
	tic; SetRIXS(minmax_E); toc;
	fprintf('\n## SAVING CALCULATED RIXS: ... \n');
	SaveCalcRixs();
	%--------------------------------------------
	fprintf('\nFINISHED... \n');
end
%==========================================================
% PREPARE DATA FROM CH CORE-HOLE SITES:
% Collect all the extended points for the reduced Bruilloin zone and
% their indexes mapped to the kPoints
function kPointsFull = GetFull_kPoints(N)
	dk = 2*pi/N;
	kx = dk*(-3*N/2:3*N/2);
	ky = kx;
	nn = length(kx);
	for iy = 1:nn
		i = (iy-1)*nn+1;
		kPointsFull(i:i+nn-1,1) = kx';
		kPointsFull(i:i+nn-1,2) = ky(iy);
	end
end
function [V_CH0,MAP_CH0] = GetBaseCH0(H_CH0)
	global GL
	
	% allocate space
	nk = size(GL.kPoints,1);
	nE = size(H_CH0,1);			% # of electorns, total (per spin)
	V_CH0    = zeros(nE,nE, 2); % storage for column eigen-vectors
	MAP_CH0 = zeros(nE, 6, 2);	% storage for [E,kPoint_index,band_index,  E-H index,kx,ky]
	iBands = (1:16)';
	% go through every k-point
	jj = 1:16;	
	for ik = 1:nk
		% k-point
		k = GL.kPoints(ik,:);
		% spin up/down
		for spin = 1:2
			[V,E] = eig( H_CH0(jj,jj,spin) ); E = diag(E);
			V_CH0(jj,jj,spin)  = V;			% eigen vectors
			MAP_CH0(jj,1,spin) = E;			% energies
			MAP_CH0(jj,2,spin) = ik;		% kPoint index
			MAP_CH0(jj,3,spin) = iBands;	% band index
			MAP_CH0(jj,5,spin) = k(1);		% k_x
			MAP_CH0(jj,6,spin) = k(2);		% k_y
		end
		% iteration
		jj = jj + 16;
	end	
	% sort everything with increasing energy
	for spin = 1:2
		MAP_CH0(:,1,spin) = ReNormalize(MAP_CH0(:,1,spin));	
		[MAP_CH0(:,:,spin), ind] = sortrows(MAP_CH0(:,:,spin), 1);
		V_CH0(:,:,spin) = V_CH0(:,ind,spin);
	end
	% set E-H index, used in RIXS table
	MAP_CH0(:,4, 1) = 1:2:2*nE-1;
	MAP_CH0(:,4, 2) = 2:2:2*nE;
end
function [Beta_occ, CoF, Beta_unc, Z, CH_iPoint] = SetBetaMatrices(inv_V_CH0,MAP_CH0, file_CH, CH)
	% load calculated core-hole Hamiltonian into CH1:
	load( file_CH );
	if CH1.CH ~= CH
		disp('ERR: Core hole input data file mismatch.');
	end
	[V_up,~] = eig(CH1.H_eg(:,:,1));
	[V_dn,~] = eig(CH1.H_eg(:,:,2));
	BetaFull_up = inv_V_CH0(:,:,1) * V_up;
	BetaFull_dn = inv_V_CH0(:,:,2) * V_dn;
	
	% allocate space
	nE = size(MAP_CH0,1);
	nF = nE / 8;	
	Beta_occ = zeros(   nF,nF,2);
	CoF      = zeros(   nF,nF,2);
	Beta_unc = zeros(nE-nF,nF,2);
	
	% extract return values
	Beta_occ(:,:,1) = BetaFull_up(1:nF,1:nF);
	Beta_occ(:,:,2) = BetaFull_dn(1:nF,1:nF);
	
	Z = [det(Beta_occ(:,:,1)), det(Beta_occ(:,:,2))];
	
	CoF(:,:,1) = ( Z(1) .* inv(Beta_occ(:,:,1)) ).';
	CoF(:,:,2) = ( Z(2) .* inv(Beta_occ(:,:,2)) ).';
	
	Beta_unc(:,:,1) = BetaFull_up(nF+1:nE,1:nF);
	Beta_unc(:,:,2) = BetaFull_dn(nF+1:nE,1:nF);
	
	CH_iPoint = CH1.CH_site;
end
function SetGlobals(N)
	% starting with blank GL and blank TMP:
	global GL
	
	if mod(N,4)~=0 || N<4
		error('ERR: N must be divisible by 4, and N >= 4.');
	end

	GL.path = sprintf('_N%02i_data_',N);
	
	% load calculated base Hamiltonian into CH0:
	load( sprintf('%s/_RIXS_/N%02i_CH0.mat', GL.path, N) );
	
	GL.N = N;
	GL.C = CH0.C;
	
	GL.dir_tables = [GL.path,'/_tables_'];
	GL.kPoints = CH0.kPoints;
	
	% k-points for extended Brillouin zone
	GL.kPointsFull = GetFull_kPoints(N);

	% return data from base Hamiltonian (CH0) ordered in increasing energy
	[V_CH0,MAP_CH0] = GetBaseCH0(CH0.H_eg);
	GL.MAP_CH0 = MAP_CH0;
	
	% create inverse once for re-occuring multiplication
	inv_V_CH0 = zeros(size(V_CH0));
	inv_V_CH0(:,:,1) = inv(V_CH0(:,:,1).').';
	inv_V_CH0(:,:,2) = inv(V_CH0(:,:,2).').';

%{
TFS NOTES:
	TMP.Beta_occ is redundant since we need TMP.CoF,
	but keep for testing, for now!
%}
	
	% prepare beta matrices
	nCH = 8;
	nE = size(CH0.H_eg,1);
	nF = nE / 8;								% 8 -> electron filling
	GL.Beta_occ = zeros(   nF, nF, 2, nCH);	% 2 -> spins
	GL.CoF      = zeros(   nF, nF, 2, nCH);	% 2 -> spins
	GL.Beta_unc = zeros(nE-nF, nF, 2, nCH);	% 2 -> spins
	GL.Z = zeros(nCH,2);						% 2 -> spins
	GL.CH_iPoint = zeros(nCH, 2);				% 2 -> ix,iy
	for CH = 1:nCH
		file_CH = sprintf('%s/_RIXS_/N%02i_CH%i.mat', GL.path,N,CH);
		[Beta_occ, Cof, Beta_unc, Z, CH_iPoint] = SetBetaMatrices(inv_V_CH0,MAP_CH0, file_CH, CH);
		GL.Beta_occ(:,:,:, CH) = Beta_occ;
		GL.CoF(:,:,:, CH) = Cof;
		GL.Beta_unc(:,:,:, CH) = Beta_unc;
		GL.Z(CH,:) = Z;
		GL.CH_iPoint(CH,:) = CH_iPoint;
	end
	
	%{-
	% adding distortions at Mn4+ sites:
	ds = 0.01871;
	dd = [  0,0; ...
			 ds,-ds;...
			0,0; ...
			-ds, ds;...
			0,0; ...
			 ds,-ds;...
			0,0; ...
			-ds, ds ];
	GL.CH_iPoint = GL.CH_iPoint + dd;
	%}
end

function nn = GetNN(dk)
	a = ceil( (-3*pi - dk(1))*2/pi );
	b = floor(( 3*pi - dk(1))*2/pi );
	n = a:b;
	nR = length(n);
	nn = zeros(nR,3);
	for i = 1:nR
		a = ceil( (-3*pi + dk(2))/pi + n(i)/2 );
		b = floor(( 3*pi + dk(2))/pi + n(i)/2 );		
		nn(i,:) = [n(i),a,b];
	end
end
function [q,iq] = GetQ(dk,n,n2)
	global GL
	
	q = dk + [n*pi/2, (n-2*n2)*pi/2];
	% indexes
	kk = (GL.kPointsFull(:,1) - q(1)).^2 + (GL.kPointsFull(:,2) - q(2)).^2;
	iq = find(kk < 1e-10);	
	q = GL.kPointsFull(iq,:);	% better quality q-value
end
function SetRIXS(minmax_E)
	global GL
	
	nE = size(GL.MAP_CH0,1);
	nF = nE / 8;
	nk = size(GL.kPoints, 1);
%	nRows = 8*nk*nk * 14*2 * 2;		% same as: (nE-nF)*nF * 8 * 2
	iRow = 1;
	%fprintf('    * Calculating RIXS table with %i rows...\n', nRows);
	fprintf('    * Calculating RIXS table...\n');
	
	% RixsData Columns:
	% 1-hole id, 2-electron id, 3-intensity, 4-energy
	% 5:6-momentum, 7-spin, 8-indexes of k-points
%	RixsData = zeros(nRows,8);
	SumIrixs = zeros(size(GL.kPointsFull,1), 2);
	
%	DK = [0,0;  pi,0;  0,pi;  pi,pi;  -pi/2,-pi/2;  pi/2,-pi/2;  -pi/2,pi/2;  pi/2,pi/2];
	mapOCC = GL.MAP_CH0(   1:nF,:, :);
	mapUNC = GL.MAP_CH0(nF+1:nE,:, :);
	rowUNC = zeros(8,nF, 2);
	% amplitude calculation terms, for every {ke,le,kh,lh,  spin,K}:
	PH = zeros(8,1);							% phase
	X1 = zeros(8,1);							% one CH excitation
	X0 = conj(GL.Z(:,1)) .* conj(GL.Z(:,2));	% no excitation
	for iUNC = 1:(nE-nF)
		% extract rows for use with CoF
		for CH = 1:8
			rowUNC(CH,:, 1) = GL.Beta_unc(iUNC,:, 1,CH);
			rowUNC(CH,:, 2) = GL.Beta_unc(iUNC,:, 2,CH);
		end
		
		% populate RIXS table
		for iOCC = 1:nF			
			% energy
			w = mapUNC(iUNC,1, 1) - mapOCC(iOCC,1, 1);		% spin makes no difference, so spin=1, 3-rd argument
			% process only below certain energy
			if w <= minmax_E(2)
				for spin = 1:2
					% excitation contribution
					% example for spin up: Z_dn * DETERMINANT^[Beta(SwapedRow)_up]
					spin_neg = mod(spin,2)+1;
					for CH = 1:8
						X1(CH,1) = GL.Z(CH,spin_neg) .* (rowUNC(CH,:, spin) * (GL.CoF(iOCC,:, spin,CH)).');
					end

					% k-indexes, band-indexes
					ik_h = mapOCC(iOCC,2, spin);	ik_e = mapUNC(iUNC,2, spin);
					%il_h = mapOCC(iOCC,3, spin);	il_e = mapUNC(iUNC,3, spin);
					iH   = mapOCC(iOCC,4, spin);	iE   = mapUNC(iUNC,4, spin);
					% momentum change in 1-st BZ
					dk = GL.kPoints(ik_e,:) - GL.kPoints(ik_h,:);

					% collect all integers for receiprocal lat.vectors in
					% extended BZ (-3pi <=> 3pi)
					nn = GetNN(dk);
					for i = 1:size(nn,1)
						for n2 = nn(i,2):nn(i,3)
							% momentum change in the extended BZ
							[q, iq] = GetQ(dk, nn(i,1),n2);
							% phase
							PH = exp(1i * (GL.CH_iPoint * q') );	% same as PH = exp(-1i * (GL.CH_iPoint * (-q)') );
							% amplitude for all CH
							A = sum(PH .* X1 .* X0);

							% save data for integrated k-map
							SumIrixs(iq, spin) = SumIrixs(iq, spin) + (A*conj(A));

							%{-
							%  collecting data from main diagonal
							if abs(q(2)-q(1)) < 1e-10
								% one data row for rixs table
								RixsRow = [
									iH, ...			% hole id
									iE, ...			% electron id
									A*conj(A), ...	% rixs intensity
									w, ...			% w energy
									q(1), ...		% q_x momentum
									q(2), ...		% q_y momentum
									spin, ...		% spin
									iq ...			% q-index
									];
								RixsData(iRow,:) = RixsRow;
								iRow = iRow+1;
							end
							%}

						end  % n2
					end  % i	
				end  % spin			
			end  % if w...
		end  % iOCC
		
		percent = 100*iUNC/(nE-nF);
		if mod(percent,25)==0
			fprintf('    * RIXS table %g%% completed.\n', percent);
		end
	end  % iUNC
		
	% set new k-points for extended BZ
	GL.kPoints = GL.kPointsFull;
	% set integrated intensity extended map
	GL.RIXS_SUM = SumIrixs;
	
iRow
GL.RIXS = RixsData;
	
	%--------------------------------------------
	% Averaging w.r.t. qx <-> -qx:
	% create index swap array
	nk = size(GL.kPoints,1);
	ind_swap = (1:nk)';
	nSet = GL.N*3+1;
	for iy = 1:nSet
		jj = ((iy-1)*nSet) + (1:nSet)';
		ind_swap(jj,1) = flipud(jj);
	end	
	% combined rixs, averaged over kx <-> -kx
	GL.RIXS_SUM_AVG = ( GL.RIXS_SUM + GL.RIXS_SUM(ind_swap,:)) / 2;
	%--------------------------------------------
	
	% cleanup GL
	GL = rmfield(GL, 'kPointsFull');
	GL = rmfield(GL, 'Beta_occ');
	GL = rmfield(GL, 'CoF');
	GL = rmfield(GL, 'Beta_unc');
	GL = rmfield(GL, 'Z');
	GL = rmfield(GL, 'CH_iPoint');
end
%==========================================================
% This is  necessary to eliminate 14/15-th significant
% digit discrepancy in degenerate values
function EE = ReNormalize(E)
	[EE, ind] = sortrows(E,1);
	dig_eps = 1000*eps(EE(1));
	
	igr = 1;
	groups(igr,1) = 1;
	val = EE(1);
	for i = 2:size(EE,1)
		% if values are not close
		if abs(EE(i) - val) > dig_eps
			val = EE(i);
			groups(igr,2) = i-1;
			igr = igr+1;
			groups(igr,1) = i;
		end
	end
	groups(igr,2) = i;
	
	% fixing values
	for i = 1:size(groups,1)
		a = groups(i,1);
		b = groups(i,2);
		avg = mean( EE(a:b,1) );
		EE(a:b,1) = avg;
	end
	
	EE = sortrows([EE, ind],2);
	EE = EE(:,1);
end
%==========================================================
% SAVING/LOADING:
function SaveCalcRixs()
	% ACCESS to save
	global GL
	
	save(sprintf('%s/_RIXS_/N%02i_CalcRixs_Ext.mat',GL.path,GL.N), 'GL');
end
%==========================================================
